part of selene_json;

/// A utility class to serialize and deserialize JSON objects.
class JsonClient {
  JsonProvider _provider;
  JsonOptions _defaultOptions;

  JsonClient({JsonProvider provider, JsonOptions defaultOptions}) {
    _provider = provider ?? new _DefaultJsonProvider();
    _defaultOptions = defaultOptions ?? new JsonOptions();
  }

  /// Removes all entries from a [Map] that meet any of the following conditions:
  ///
  /// a) Key is null.
  ///
  /// b) Value is null.
  ///
  /// c) Key is empty.
  Map<String, dynamic> removeNulls(Map<String, dynamic> input) {
    Map<String, dynamic> newMapLocal = {};
    for (var entry in input.entries) {
      if (entry.key != null && entry.key.isNotEmpty && entry.value != null) {
        if (entry.value is Map<String, dynamic>) {
          newMapLocal[entry.key] = removeNulls(entry.value);
        } else
          newMapLocal[entry.key] = entry.value;
      }
    }
    return newMapLocal;
  }

  /// Serializes a JSON object.
  String serialize(Map<String, dynamic> inputModel,
      [JsonOptions options = null]) {
    Map<String, dynamic> newMap = inputModel;
    if ((options ?? _defaultOptions).removeNulls) newMap = removeNulls(newMap);

    var serialized = _provider.serialize(inputModel, options);
    return serialized;
  }

  /// De-serializes a JSON object.
  Map<String, dynamic> deserialize(String inputString,
      [JsonOptions options = null]) {
    return _provider.deserialize(inputString, options);
  }
}
