part of selene_json;

/// An abstract class for providing JSON-related services.
abstract class JsonProvider {
  Map<String, dynamic> deserialize(String jsonInput, JsonOptions options);

  String serialize(Map<String, dynamic> input, JsonOptions options);
}

/// A data object for storing JSON options.
class JsonOptions {
  bool throwOnFailure;
  bool removeNulls;

  JsonOptions({this.throwOnFailure = false, this.removeNulls = true});
}

/// The default implementation of [JsonProvider], using dart:convert.
class _DefaultJsonProvider extends JsonProvider {
  JsonEncoder _encoder = new JsonEncoder();
  JsonDecoder _decoder = new JsonDecoder();

  _DefaultJsonProvider();

  @override
  Map<String, dynamic> deserialize(String jsonInput, JsonOptions options) {
    try {
      return _decoder.convert(jsonInput);
    } on Error {
      if (options.throwOnFailure) rethrow;
      return null;
    } on Exception {
      if (options.throwOnFailure) rethrow;
      return null;
    }
  }

  @override
  String serialize(Map<String, dynamic> input, JsonOptions options) {
    try {
      return _encoder.convert(input);
    } on Error {
      if (options.throwOnFailure) rethrow;
      return null;
    } on Exception {
      if (options.throwOnFailure) rethrow;
      return null;
    }
  }
}
