part of selene;

/// A generic channel in Discord.
abstract class Channel extends SnowflakeEntity {
  /// The type of this channel.
  ChannelType type;

  /// Creates a new [Channel] object using the specified data.
  Channel(Map<String, dynamic> model, Client client) : super(model, client) {
    if (model['type'] != null) {
      switch (model['type']) {
        case 0:
          type = ChannelType.GuildText;
          break;
        case 1:
          type = ChannelType.DM;
          break;
        case 2:
          type = ChannelType.GuildVoice;
          break;
        case 3:
          type = ChannelType.GroupDM;
          break;
        case 4:
          type = ChannelType.GuildCategory;
          break;
        default:
          throw new ModelUpdateException(this,
              'Received invalid channel type from Discord. Is this library out-of-date?');
      }
    }
  }

  factory Channel._cached(Map<String, dynamic> model, Client client,
      [Guild guild = null]) {
    return client.state.resolveChannel(parseInt(model['id']));
  }

  /// Generates a [Channel] based off of the supplied data.
  static Channel _create(Map<String, dynamic> model, Client client,
      [Guild guild = null]) {
    var type = _getType(model);
    if (type == null) return null;

    switch (_getType(model)) {
      case ChannelType.GuildText:
        return new GuildTextChannel._create(model, guild, client);
      case ChannelType.DM:
        return new DMChannel._create(model, client);
      case ChannelType.GuildVoice:
        return new GuildVoiceChannel._create(model, guild, client);
      case ChannelType.GuildCategory:
        return new CategoryChannel._create(model, guild, client);
      default:
        return null;
    }
  }

  static ChannelType _getType(Map<String, dynamic> model) {
    if (model['type'] == null) return null;
    switch (model['type']) {
      case 0:
        return ChannelType.GuildText;
      case 1:
        return ChannelType.DM;
      case 2:
        return ChannelType.GuildVoice;
      case 3:
      // Ignore Group DMs
        return null;
      case 4:
        return ChannelType.GuildCategory;
      default:
        return null;
    }
  }

  /// Modifies this channel using the specified method.
  ///
  /// Client must have the `MANAGE_CHANNELS` permission.
  Future modify({String name,
    int position,
    String topic,
    bool isNsfw,
    int bitrate,
    int userLimit,
    List<PermissionOverwrite> overwrites,
    CategoryChannel category}) async {
    var props = fillNotNullProperties({
      'name': name,
      'position': position,
      'topic': topic,
      'nsfw': isNsfw,
      'bitrate': bitrate,
      'user_limit': userLimit
    });
    if (overwrites != null)
      props['permission_overwrites'] = overwrites.map((o) => o.build());
    if (category != null) props['parent_id'] = category.id;

    await client._rest.makeNewRequest('/channels/${id}', 'PATCH', props);
  }
}

/// A type of channel.
enum ChannelType { GuildText, GuildVoice, GuildCategory, DM, GroupDM }
