part of selene;

/// A channel belonging to a guild in Discord.
abstract class GuildChannel extends Channel {
  /// The [Guild] this channel belongs to.
  Guild guild;

  /// The parent category of this channel.
  ///
  /// Null if it does not have a parent.
  CategoryChannel get category {
    if (_parentId == null) return null;
    return guild.channels[_parentId];
  }

  int _parentId;

  /// The sorting position of this channel in the channel listing.
  int sortingPosition;

  /// The name of this channel.
  String name;

  /// The permission overwrites in this channel.
  List<PermissionOverwrite> permissionOverwrites = [];

  /// Whether this channel is NSFW.
  bool isNsfw;

  /// Whether this channel is not NSFW.
  bool get isNotNsfw => !isNsfw;

  void _parseOverwrites(List<dynamic> overwrites) {
    for (var jsonOverwrite in overwrites ?? []) {
      if (!permissionOverwrites.any((p) =>
      p.idAsString() == jsonOverwrite['id']))
        permissionOverwrites.add(
            new PermissionOverwrite._create(jsonOverwrite, client));
    }
  }

  GuildChannel._create(Map<String, dynamic> model, this.guild, Client client)
      : super(model, client) {
    _update(model);

    guild.channels[id] = this;
    client.state.guildChannels[id] = this;
  }

  factory GuildChannel._cacheOrNew(Map<String, dynamic> model, Guild guild,
      Client client) {
    var parsedId = parseInt(model['id']);
    if (client.state.guildChannels.containsKey(parsedId))
      return client.state.guildChannels[parsedId];
    else {
      var type = Channel._getType(model);
      if (type == ChannelType.GuildText) {
        return new GuildTextChannel._create(model, guild, client);
      } else if (type == ChannelType.GuildVoice) {
        return new GuildVoiceChannel._create(model, guild, client);
      } else if (type == ChannelType.GuildCategory) {
        return new CategoryChannel._create(model, guild, client);
      } else {
        throw new StateError(
            'Received invalid channel type during GuildChannel constructor');
      }
    }
  }

  @mustCallSuper
  void _update(Map<String, dynamic> model) {
    name = model['name'] ?? name;
    isNsfw = model['nsfw'] ?? isNsfw;
    sortingPosition = model['position'] ?? sortingPosition;

    _parentId = parseInt(model['parent_id']);

    _parseOverwrites(model['permission_overwrites']);
  }

  @override
  String toString() => "${name} (of guild ${guild.name})";

  /// Deletes/closes this channel.
  ///
  /// This cannot be undone. Deleting a category will not delete it's children.
  Future delete() => client._rest.makeNewRequest('/channels/${id}', 'DELETE');
}
