part of selene;

/// An overwrite for channel permissions for a role or user.
class PermissionOverwrite extends SnowflakeEntity {
  PermissionOverwrite._create(Map<String, dynamic> model, Client client)
      : super(model, client) {
    switch (model['type']) {
      case 'role':
        type = OverwriteType.Role;
        break;
      case 'member':
        type = OverwriteType.Member;
        break;
      default:
        throw new ModelUpdateException(
            this, 'Permission overwrite is of invalid type.');
    }
    _update(model);
  }

  /// The type of this overwrite.
  OverwriteType type;

  /// The permission bit set which this overwrite allows.
  int allowed;

  /// The permission bit set which this overwrite denies.
  int denied;

  void _update(Map<String, dynamic> model) {
    allowed = model['allow'];
    denied = model['deny'];
  }

  Map<String, dynamic> build() {
    return <String, dynamic>{
      'id': id,
      'type': type == OverwriteType.Role ? 'role' : 'member',
      'allow': allowed,
      'deny': denied
    };
  }
}

/// A type of overwrite.
enum OverwriteType { Role, Member }
