part of selene;

/// Represents an emote in Discord.
///
/// This Emote can be a standard Emote or a guild Emote.
abstract class Emote extends SnowflakeEntity {
  /// Creates a new [Emote].
  Emote(Map<String, dynamic> data, Client client) : super(data, client) {
    name = data['name'];
  }

  /// The name of this emote.
  String name;

  /// Determines the type and returns a new [Emote].
  static Emote fromJson(Map<String, dynamic> model, Client client,
      [Guild guild = null]) {
    if (model['id'] == null) {
      return new StandardEmote(model, client);
    }
    return new GuildEmote(model, guild, client);
  }
}

/// Represents a custom emote on a [Guild].
class GuildEmote extends Emote {
  /// Creates a new [GuildEmote].
  GuildEmote(Map<String, dynamic> data, this.guild, Client client)
      : super(data, client) {
    for (var jsonRole in data['roles'] ?? []) {
      if (guild.roles.any((r) => r.id == parseInt(jsonRole ?? ''))) {
        roles.add(guild.roles
            .firstWhere((r) => r.id == parseInt(jsonRole ?? '')));
      }
    }

    if (data['user'] != null) {
      author = new User._cached(data['user'], client);
    }

    _update(data);
  }

  /// The guild that owns this emote.
  Guild guild;

  /// A list of roles that this emote is whitelisted to.
  List<GuildRole> roles = [];

  /// The user who created this emote.
  User author;

  /// Whether this emote must be wrapped in colons.
  bool requiresColons;

  /// Whether this emote doesn't have to be wrapped in colons.
  bool get doesNotRequireColons => !requiresColons;

  /// Whether this emote is managed by an integration.
  bool isManaged;

  /// Whether this emote is not managed by an integration.
  bool get isNotManaged => !isManaged;

  /// Whether this emote is animated.
  bool isAnimated;

  /// Whether this emote is not animated.
  bool get isNotAnimated => !isAnimated;

  void _update(Map<String, dynamic> model) {
    requiresColons = model['require_colons'] ?? requiresColons;
    isManaged = model['managed'] ?? isManaged;
    isAnimated = model['animated'] ?? isAnimated;
  }
}

/// Represents a standard Discord Emote.
class StandardEmote extends Emote {
  /// Creates a new [StandardEmote].
  StandardEmote(Map<String, dynamic> data, Client client) : super(data, client);
}
