part of selene;

/// A generic Discord entity.
abstract class Entity<TId> {
  /// The ID of this entity.
  TId id;

  /// The client that this entity is attached to.
  ///
  /// Usually this is the client that instantiated or retrieved this entity.
  Client client;

  /// Creates a new blank entity using the specified [Client] root object.
  Entity(this.client);

  /// Converts this ID type to a String value.
  String idAsString();

  @override
  String toString() =>
      "Discord entity of type '${runtimeType
          .toString()}' (with ID ${idAsString()})";
}

/// A Discord entity with an integer-based ID.
abstract class SnowflakeEntity extends Entity<int> {
  SnowflakeEntity(Map<String, dynamic> model, Client client) : super(client) {
    id = parseInt(model['id']);
  }

  /// The ID of this entity as a Snowflake object.
  Snowflake get snowflakeId => new Snowflake.parseInt(id);

  @override
  String idAsString() => id.toString();
}

/// A Discord entity with a String-based ID.
abstract class StringEntity extends Entity<String> {
  StringEntity(Map<String, dynamic> model, Client client) : super(client) {
    id = model['id'];
  }

  @override
  String idAsString() => id;
}