part of selene;

/// A member of a guild in Discord.
class GuildMember extends SnowflakeEntity {
  /// The internal [User] object that this member wraps.
  User user;

  /// The guild that this member is a part of.
  Guild guild;

  /// The nickname of this member.
  String nickname;

  /// The roles that this member has.
  List<GuildRole> roles = [];

  /// When this member joined the server.
  DateTime joined;

  /// Whether this member is currently deafened.
  bool isDeafened = false;

  /// Whether this member is currently muted.
  bool isMuted = false;

  /// Whether this member is currently not deafened.
  bool get isNotDeafened => !isDeafened;

  /// Whether this member is currently not muted.
  bool get isNotMuted => !isMuted;

  /// Updates this member's roles from a provided JSON array.
  void _updateRoles(List<dynamic> roles) {
    for (var jsonRole in roles ?? []) {
      var parsedId = parseInt(jsonRole);
      if (this.guild != null) {
        if (!this.roles.any((r) =>
        r.id == parsedId)) // Ignore it if we already have this role (updating)
          this.roles.add(
              guild.roles.firstWhere((r) => r.id == parsedId));
      }
    }
  }

  /// Creates a new [GuildMember].
  GuildMember._create(Map<String, dynamic> model, this.guild, Client client)
      : super(model, client) {
    id = parseInt((model['user']['id']) ?? '');

    if (model['guild_id'] != null) {
      guild = client.state.guilds[parseInt(model['guild_id'])];
    }

    user = new User._cachedOrNew(model['user'], client);

    _updateRoles(model['roles']);

    joined = DateTime.tryParse(model['joined_at'] ?? '') ?? joined;
    _update(model);

    client.state.members[id] = this;
    guild.members[id] = this;
  }

  factory GuildMember._cachedOrNew(Map<String, dynamic> model, Guild guild,
      Client client) {
    var parsedId = parseInt((model['user']['id']) ?? '');
    if (client.state.members.containsKey(parsedId)) {
      return new GuildMember._cached(model, client);
    } else {
      return new GuildMember._create(model, guild, client);
    }
  }

  factory GuildMember._cached(Map<String, dynamic> model, Client client) {
    var parsedId = parseInt((model['user']['id']) ?? '');
    return client.state.members[parsedId];
  }

  @override
  String toString() => "${user}, as a member of ${guild.name}";

  void _update(Map<String, dynamic> model) {
    nickname =
    model['nick']; // No null coalescing because being sent null means nick removed
    isDeafened = model['deaf'] ?? isDeafened;
    isMuted = model['mute'] ?? isMuted;
  }
}
