part of selene;

/// Handles complex embed-building for Discord messages.
class EmbedBuilder {
  /// Title of this embed.
  String title = null;

  /// Type of this embed. [rich] by default.
  String type = "rich";

  /// Description of this embed.
  String description = null;

  /// URL of this embed.
  String url = null;

  /// Colour of this embed.
  int colour = null;

  /// Author of this embed.
  EmbedAuthorBuilder author = null;

  /// Fields in this embed.
  List<EmbedFieldBuilder> fields = [];

  /// The footer of this embed.
  EmbedFooterBuilder footer = null;

  /// Builds this embed into a JSON map.
  Map<String, dynamic> build() {
    var map = <String, dynamic>{};

    if (title != null) map['title'] = title;
    if (type != null) map['type'] = type;
    if (description != null) map['description'] = description;
    if (url != null) map['url'] = url;
    if (colour != null) map['color'] = colour;
    if (author != null) map['author'] = author.build();
    if (footer != null) map['footer'] = footer.build();
    if (fields != null) {
      var arr = <Map<String, dynamic>>[];
      fields.forEach((fBuilder) {
        arr.add(fBuilder.build());
      });
      map['fields'] = arr;
    }

    return map;
  }

  /// Adds a new field to this field map.
  void addField(String name, String value, [bool inline = true]) =>
      fields.add(new EmbedFieldBuilder(name, value, inline));
}

/// Handles creating authors for embeds in Discord
class EmbedAuthorBuilder {
  /// Name of this author.
  String name = null;

  /// URL of this author.
  String url = null;

  /// Icon URL of this author.
  String iconUrl = null;

  /// Proxied Icon URL of this author.
  String proxyIconUrl = null;

  /// Builds this author into a JSON map.
  Map<String, dynamic> build() {
    var map = <String, dynamic>{};

    if (name != null) map['name'] = name;
    if (url != null) map['url'] = url;
    if (iconUrl != null) map['icon_url'] = iconUrl;
    if (proxyIconUrl != null) map['proxy_icon_url'] = proxyIconUrl;

    return map;
  }

  EmbedAuthorBuilder(this.name, {this.iconUrl, this.url, this.proxyIconUrl});

  EmbedAuthorBuilder.fromUser(User user) {
    name = user.username;
    iconUrl = user.getAvatarUrl();
  }
}

/// Handles creating fields for embeds in Discord
class EmbedFieldBuilder {
  /// Name of this field.
  String name = null;

  /// Value of this field.
  String value = null;

  /// Whether or not this field should display inline.
  bool inline = true;

  EmbedFieldBuilder(this.name, this.value, [this.inline = true]);

  Map<String, dynamic> build() {
    var map = <String, dynamic>{};

    if (name != null) map['name'] = name;
    if (value != null) map['value'] = value;
    if (inline != null) map['inline'] = inline;

    return map;
  }
}

/// Handles creating footers for embeds in Discord
class EmbedFooterBuilder {
  /// The text of this footer.
  String text = null;

  /// The icon URL of this footer.
  String iconUrl = null;

  /// The proxied icon URL of this footer.
  String proxyIconUrl = null;

  EmbedFooterBuilder(this.text, {this.iconUrl, this.proxyIconUrl});

  /// Builds this embed footer into a JSON map.
  Map<String, dynamic> build() {
    var map = <String, dynamic>{};

    if (text != null) map['text'] = text;
    if (iconUrl != null) map['icon_url'] = iconUrl;
    if (proxyIconUrl != null) map['proxy_icon_url'] = proxyIconUrl;

    return map;
  }
}