part of selene;

/// A message received or being sent to the WebSocket.
class _WSFrame implements _Serializable {
  /// The data of this frame (the `d` property)
  ///
  /// This can either be a number, a string or a [Map<String, dynamic>] JSON map.
  dynamic data;

  /// The sequence number of this frame (the `s` property)
  ///
  /// Can only be received, and only on OP Code 0 - Dispatch.
  int sequenceNumber;

  /// The name of the event in this payload.
  ///
  /// Can only be received, and only on OP Code 0 - Dispatch.
  String eventName;

  /// The operation code (OP Code) of this payload.
  int opCode;

  /// The direction of this payload.
  FrameDirection direction;

  /// Converts a JSON map to a [_WSFrame].
  void update(Map<String, dynamic> model) {
    data = model['d'];
    sequenceNumber = model['s'];
    eventName = model['t'];
    opCode = model['op'];
  }

  /// Creates a blank [_WSFrame].
  _WSFrame();

  /// Converts a [_WSFrame] to a JSON map that can be sent.
  Map<String, dynamic> build() {
    var map = <String, dynamic>{};
    map['d'] = data;
    map['op'] = opCode;
    // eventName and sequenceNumber can only be received
    return map;
  }
}

/// The direction in which a frame is being sent.
enum FrameDirection {
  /// A frame that can only be sent to Discord.
  Outbound,

  /// A frame that can only be received.
  Inbound,

  /// A frame that can be received or sent.
  Bidirectional
}
